import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * The ball of the game! Pretty important for Pong, to be honest.
 * 
 * This actor handles the ball and score (backend, not visual!!!). 
 */

public class Ball extends Actor
{
    public int xV = 6;
    public int counter;
    private boolean firstRun;
    public int yV = 6;
    public int ballX = 0;
    public int ballY = 0;
    public static int scoreP1 = 0;
    public static int scoreP2 = 0;
    static GreenfootSound practice = new GreenfootSound("sounds/Music/mus_practice.mp3");
    static GreenfootSound  result = new GreenfootSound("sounds/Music/mus_result.mp3"); 

    
    /**
     * Sets the image of the ball, the size/scale of the ball 
     * and resets static ints scoreP1 and scoreP2.
     * 
     * 
     * Runs upon intialization of the actor.
     */
    public Ball()
    {
        GreenfootImage image = getImage();
        image.scale(image.getWidth() + 5, image.getHeight() + 5);
        setImage(image);
        scoreP1 = 0;
        scoreP2 = 0;
    }
    
    /**
     * The main code for the ball- it looks more complex than it really is, trust me.
     * 
     * On first run (indicated by bool firstRun), the ball will play the song 
     * Practice on loop.
     * 
     * After that, it'll check to make sure that int Counter is less than zero, which
     * is used in order to keep the ball in the middle when somebody gets a point.
     * If not, it'll use method tryBounce (more in its comment).
     * If so, it'll reduce Counter by 1, set both horizontal and vertical velocity to zero.
     * 
     * X Velocity: xV.
     * Y Velocity: yV.
     */
    public void act()
    {
        if(!firstRun)
        {
            practice.playLoop();
            firstRun = true;
        }
        if(counter < 0)
        {
            setLocation(getX() + xV, getY() + yV);
            updateLocation();
            tryBounce();
        }
        else
        {
            counter = counter - 1;
            xV = 0;
            yV = 0;
            updateLocation();
            if(!(counter>=0))
            {
                xV = 6;
                yV = 6;
            }
        }
        // getImage().rotate(30); <-- Unused ball rotation code!! Remove comment to enable.//
    }
    
    /**
     * Updates the current x location of the ball.
     * 
     * Used for calculating score, as that function uses the previous 
     * position of the ball before resetting to the middle.
     * 
     * 
     * Absolutely a workaround until I figure out how to structure my code better LOL
     */
    public void updateLocation()
    {
        ballX = getX();
    }
    
    /**
     * Checks if the conditions are met to bounce.
     * 
     * Those conditions are met if either:
     * - The ball is touching a paddle.
     * - The ball is touching the top or bottom of the screen.
     * 
     * If the former, it'll reverse the ball's x velocity.
     * If the latter, it'll reverse the ball's y velocity.
     * 
     * There is code for playing a sound upon collision, however due to
     * time constraints I couldn't do proper sound mixing, so it's been
     * commented out. The code is fully functional, though, so if you'd like
     * to enable it just uncoment the playSound methods.
     *
     * 
     * In addition, the ball checks for which side of the screen it's
     * on in order to determine which paddle it's touching.
     * 
     * This is used to prevent the fabled "Sticky Paddle" bug.
     */
    public void tryBounce()
    {
        if (getY() >= getWorld().getHeight() - 1 || getY() <= 0)
        {
            yV = -yV;
            // Greenfoot.playSound("sounds/Effects/graze_low.wav"); 
        }
        if ( isTouching(Paddle.class) )
            {
               if (xV > 0 && getX() > getWorld().getWidth() / 2)
               {   
                  xV = (-1) * (xV + 1); 
                  // Greenfoot.playSound("sounds/Effects/graze.wav"); 
               }
               else if ( xV < 0 && getX() < getWorld().getWidth() / 2)
               {
                  xV = (-1) * (xV - 1);
                  // Greenfoot.playSound("sounds/Effects/graze.wav");
               }
            }
        if (getX() >= getWorld().getWidth() - 1 || getX() <= 0)
        {
            if (!(isTouching(Paddle.class)))
            {
                setLocation((getWorld().getWidth() / 2),(getWorld().getHeight() / 2));
                getWorld().getHeight();
                GreenfootSound  sound = new GreenfootSound("sounds/Effects/hurt.wav"); 
                sound.setVolume(100); 
                sound.play();
                tryScore();
                counter = 60;
            }
        }
    }
    
    /**
     * This code controls the score!
     * 
     * Looks at what side the ball is at (greater or lesser than the center of the world)
     * and adds score accordingly.
     * 
     * If a player has at least 8 score, they win!
     * 
     * 
     * If you're looking for the visual component of score, that's handled by the Score actor.
     */
    public void tryScore()
    {
        if (ballX >= getWorld().getWidth() / 2)
        {
            scoreP1++;
        }
        else
        {
            scoreP2++;
        }
        if ((scoreP1 >= 8) || (scoreP2 >= 8))
        {
            practice.stop();
            result.setVolume(85); 
            result.play();
            Greenfoot.setWorld(new WinWorld());
        }
    }
}